/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   The event handler object is a local inner
   (nested) class of our program's constructor.
   The event handler object has access to the
   constructor's (final) local variables.
*/
public class CircleProgram_v6
{
   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram_v6()
   {
      final JFrame jf = new JFrame("Interactive Frame v6");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      final CircleState cs = new CircleState();   // Put cs and cp in the
      final CirclePanel cp = new CirclePanel(cs); // scope of the KeyListener.
      jf.getContentPane().add(cp);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.setVisible(true);

      // A (named) local inner class definition.
      class CircleListener_v6 implements KeyListener
      {
         @Override public void keyPressed(KeyEvent e)
         {
            final int keyCode = e.getKeyCode();
            if (keyCode == KeyEvent.VK_UP)
            {
               cs.yNew -= 5;  // update the state
               cp.repaint();  // notify the view
            }
            else if (keyCode == KeyEvent.VK_DOWN)
            {
               cs.yNew += 5;
               cp.repaint();
            }
            else if (keyCode == KeyEvent.VK_LEFT)
            {
               cs.xNew -= 5;
               cp.repaint();
            }
            else if (keyCode == KeyEvent.VK_RIGHT)
            {
               cs.xNew += 5;
               cp.repaint();
            }
         }
         @Override public void keyReleased(KeyEvent e){ }
         @Override public void keyTyped(KeyEvent e){ }
      }//CircleListener_v6

      jf.addKeyListener(new CircleListener_v6());
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram_v6()
      );
   }//main()
}//CircleProgram_v6

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                                CircleProgram_v6 constructor
                                                activation record of final
                                                local variables.
                                                +-------------------------+
                                                | final CirclePanel cp    |
                                                | final CircleState cs    |
                                                |                         |
      JFrame object                             |                         |
+----------------------+                        +----------------/\-------+
|                      |    List<KeyListener>                    ||
|  List<KeyListener>---|------->+------+                         || nesting link
|                      |        |      |       CircleListener_v6 ||  (closure)
|                      |        +------+        +----------------||-------+
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
GUI object that receives        |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM.                                        |                         |
                                                +-------------------------+
                                                 An object that implements
                                                 the KeyListener interface.


    KeyEvent object         CircleState object       CirclePanel object
  +-----------------+      +-----------------+      +------------------+
  |  information    |      |  int xNew       |      | CircleState cs   |
  |  from the OS    |      |  int yNew       |      |                  |
  |  about the      |      |  int xOld       |      |                  |
  |  keyboard event |      |  int yOld       |      | paintComponent() |
  +-----------------+      +-----------------+      +------------------+
                                                    GUI object that needs
                                                    to be repainted after
                                                    an event occurs.
*/
