/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   So far, the constructor's local variables used by our
   event handler were immutable. If we want an event handler
   to mutate a variable from this file, then we must treat that
   variable in a different way. A mutable variable that is
   accessible to the constructor's inner class cannot be a local
   variable of the constructor, it must be an instance variable
   of the program object.
*/
@SuppressWarnings("serial")
public class CircleProgram_v9 extends JFrame
{
   private int count = 0;  // Mutable instance variable in
                           // the scope of the KeyListener.
   public CircleProgram_v9()
   {
      super("Interactive Frame v9");
      this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      final CircleState cs = new CircleState();   // Put cs and cp in the
      final CirclePanel cp = new CirclePanel(cs); // scope of the KeyListener.
      this.getContentPane().add(cp);
      this.pack();
      this.setLocationRelativeTo(null);
      this.setVisible(true);
      //int count = 0; // Non final local variables may not be referenced by inner classes!

      this.addKeyListener(
         new KeyListener(){ // An anonymous inner class constructor.
            // Implement the three methods of the KeyListener interface.
            @Override public void keyPressed(KeyEvent e)
            {
               final int keyCode = e.getKeyCode();
               if (keyCode == KeyEvent.VK_UP)
               {
                  cs.yNew -= 5;  // update the state
                  cp.repaint();  // notify the view
               }
               else if (keyCode == KeyEvent.VK_DOWN)
               {
                  cs.yNew += 5;
                  cp.repaint();
               }
               else if (keyCode == KeyEvent.VK_LEFT)
               {
                  cs.xNew -= 5;
                  cp.repaint();
               }
               else if (keyCode == KeyEvent.VK_RIGHT)
               {
                  cs.xNew += 5;
                  cp.repaint();
               }
               CircleProgram_v9.this.count += 1; // mutate count
               System.out.printf("count = %d\n", count);
            }
            @Override public void keyReleased(KeyEvent e){ }
            @Override public void keyTyped(KeyEvent e){ }
         }//KeyListener
      );
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram_v9()
      );
   }//main()
}//CircleProgram_v9

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                         CircleProgram_v9 constructor
                                         activation record of final
                                         local variables.
                                         +-------------------------------+
                                         | final CirclePanel cp          |
                                         | final CircleState cs          |
                                         | final CircleProgram_v11 this  |
                                         |                          |    |
                                         +------------------/\------|----+
CircleProgram_v9 object                                     ||      |
(is a JFrame object)                                        ||      |
+----------------------+<-----------------------------------||------+
|                      |    List<KeyListener>               ||
|  List<KeyListener>---|------->+------+                    || nesting link
|                      |        |      |                    ||  (closure)
|  int count           |        +------+        +-----------||------------+
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
GUI object that receives        |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM.                                        |                         |
                                                +-------------------------+
                                                 An object that implements
                                                 the KeyListener interface.


    KeyEvent object         CircleState object       CirclePanel object
  +-----------------+      +-----------------+      +------------------+
  |  information    |      |  int xNew       |      | CircleState cs   |
  |  from the OS    |      |  int yNew       |      |                  |
  |  about the      |      |  int xOld       |      |                  |
  |  keyboard event |      |  int yOld       |      | paintComponent() |
  +-----------------+      +-----------------+      +------------------+
                                                    GUI object that needs
                                                    to be repainted after
                                                    an event occurs.
*/
