/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   This class builds the GUI, instantiates
   all the program objects, and makes sure
   that every object has a reference to any
   other object that it must communicate with.

   This class instantiates the JFrame that
   is the source of the key events.
*/
public class CircleProgram
{
   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      final JFrame jf = new JFrame("Interactive JFrame v1a");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      final CircleState cs = new CircleState();
      final CirclePanel cp = new CirclePanel(cs);
      jf.getContentPane().add(cp);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.setVisible(true);

      jf.addKeyListener(new CircleListener(cs, cp));
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
   }//main()
}//CircleProgram


/**
   This class has the responsibility of holding the data
   that determines the state of our graphics program.

   An event handler object of our graphics program will
   update an instance of this class whenever an appropriate
   event occurs.

   The graphics panel object of our graphics program will
   examine an instance of this object to retrieve the
   data needed to draw the program's current.
*/
class CircleState
{
   public int xNew = 300; // where the circle needs to be
   public int yNew = 300;
   public int xOld = 300; // where the circle is now
   public int yOld = 300;
}//CircleState


/**
   This class has the responsibility of updating
   what we see (view) in the graphics panel.

   The paintComponent() method will be called by the
   Event Dispatch Thread (the "GUI thread") whenever
   the GUI system decides that this panel needs to be
   redrawn.

   An instance of this class needs access to an object
   that holds the current state of out graphics program.
*/
@SuppressWarnings("serial")
class CirclePanel extends JPanel
{
   private final CircleState cs;

   public CirclePanel(CircleState cs)
   {
      this.cs = cs;
      setPreferredSize(new Dimension(600, 600));
   }

   @Override
   public void paintComponent(Graphics g)
   {
      g.setColor(Color.white);
      g.fillOval(cs.xOld, cs.yOld, 20, 20); // erase old disk
      g.setColor(Color.red);
      g.fillOval(cs.xNew, cs.yNew, 20, 20); // draw new disk
      cs.xOld = cs.xNew;
      cs.yOld = cs.yNew;
   }
}//CirclePanel


/**
   This class has the responsibility of responding to
   key events by updating the state (data) of our
   graphics program and then notifying the graphics
   part of program that it needs to redraw itself (using
   the new state information).

   The methods from the KeyListener interface will be
   called by the Event Dispatch Thread (the "GUI thread")
   whenever the program's user causes the appropriate event.
*/
class CircleListener implements KeyListener
{
   private final CircleState cs;
   private final CirclePanel cp;

   public CircleListener(CircleState cs, CirclePanel cp)
   {
      this.cs = cs;
      this.cp = cp;
   }

   // implement the KeyListener interface (three methods)
   @Override public void keyPressed(KeyEvent e)
   {
      final int keyCode = e.getKeyCode();
      if (keyCode == KeyEvent.VK_UP)
      {
         cs.yNew -= 5;   // update the state
         cp.repaint();   // notify the view
      }
      else if (keyCode == KeyEvent.VK_DOWN)
      {
         cs.yNew += 5;
         cp.repaint();
      }
      else if (keyCode == KeyEvent.VK_LEFT)
      {
         cs.xNew -= 5;
         cp.repaint();
      }
      else if (keyCode == KeyEvent.VK_RIGHT)
      {
         cs.xNew += 5;
         cp.repaint();
      }
   }
   @Override public void keyReleased(KeyEvent e){ }
   @Override public void keyTyped(KeyEvent e){ }
}//CircleListener

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+
|                      |    List<KeyListener>     CircleListener object
|  List<KeyListener>---|------->+------+        +-------------------------+
|                      |        |      |        | CircleState cs          |
|                      |        +------+        | CirclePanel cp          |
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
GUI object that receives        |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM.                                        |                         |
                                                +-------------------------+
                                                 An object that implements
    KeyEvent object                              the KeyListener interface.
  +-----------------+
  |  information    |
  |  from the OS    |      CircleState object         CirclePanel object
  |  about the      |      +-----------------+       +------------------+
  |  keyboard event |      |  int xNew       |       | CircleState cs   |
  +-----------------+      |  int yNew       |       |                  |
                           |  int xOld       |       |                  |
                           |  int yOld       |       | paintComponent() |
                           +-----------------+       +------------------+
                                                     GUI object that needs
                                                     to be repainted after
                                                     an event occurs.
*/
