/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   The important objects in our program that the listener
   object and the GUI panel object need to access now have
   public static reference variables in our program class.
   This way, the listener and the panel can access them
   without having local (instance) references to them.
   We are using a form of "global variables", which is
   not a good idea.
*/
public class CircleProgram
{
   public static CircleState cs;
   public static CirclePanel cp;

   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      final JFrame jf = new JFrame("Interactive Frame v1b");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      cs = new CircleState();
      cp = new CirclePanel();
      jf.getContentPane().add(cp);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.setVisible(true);
      jf.addKeyListener(new CircleListener());
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
   }//main()
}//CircleProgram

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                                 CircleProgram class
                                               +============================+
                                              / public static CircleState cs \
                                              \ public static CirclePanel cp /
                                              /                              \
    JFrame object                             \ public static main()         /
+----------------------+                       +============================+
|                      |    List<KeyListener>
|  List<KeyListener>---|------->+------+
|                      |        |      |         CircleListener object
|                      |        +------+        +-------------------------+
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
GUI object that receives        |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM.                                        |                         |
                                                +-------------------------+
                                                 An object that implements
    KeyEvent object                              the KeyListener interface.
  +-----------------+
  |  information    |
  |  from the OS    |      CircleState object         CirclePanel object
  |  about the      |      +-----------------+       +------------------+
  |  keyboard event |      |  int xNew       |       |                  |
  +-----------------+      |  int yNew       |       |                  |
                           |  int xOld       |       |                  |
                           |  int yOld       |       | paintComponent() |
                           +-----------------+       +------------------+
                                                     GUI object that needs
                                                     to be repainted after
                                                     an event occurs.
*/
