/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   In this version we combine the listener object
   with the program object. In other words, our
   program IS the listener object. This gives the
   listener easy access to the data and panel objects.
*/
public class CircleProgram implements KeyListener
{
   private final CircleState cs; // Compare with CircleListener, ver 1.
   private final CirclePanel cp; // or with CircleProgram, ver 1c.

   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      final JFrame jf = new JFrame("Interactive Frame v3a");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      cs = new CircleState();
      cp = new CirclePanel(cs);
      jf.getContentPane().add(cp);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.setVisible(true);
      jf.addKeyListener(this); // This CircleProgram object is the listener.
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
   }//main()


   // implement the KeyListener interface (three methods)
   @Override public void keyPressed(KeyEvent e)
   {
      final int keyCode = e.getKeyCode();
      if (keyCode == KeyEvent.VK_UP)
      {
         cs.yNew -= 5;  // update the state
         cp.repaint();  // notify the view
      }
      else if (keyCode == KeyEvent.VK_DOWN)
      {
         cs.yNew += 5;
         cp.repaint();
      }
      else if (keyCode == KeyEvent.VK_LEFT)
      {
         cs.xNew -= 5;
         cp.repaint();
      }
      else if (keyCode == KeyEvent.VK_RIGHT)
      {
         cs.xNew += 5;
         cp.repaint();
      }
   }
   @Override public void keyReleased(KeyEvent e){ }
   @Override public void keyTyped(KeyEvent e){ }
}//CircleProgram

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+
|                      |    List<KeyListener>     CircleProgram object
|  List<KeyListener>---|------->+------+        +-------------------------+
|                      |        |      |        | CircleState cs          |
|                      |        +------+        | CirclePanel cp          |
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
GUI object that receives        |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM.                                        |                         |
                                                +-------------------------+
                                                 An object that implements
    KeyEvent object                              the KeyListener interface.
  +-----------------+
  |  information    |
  |  from the OS    |      CircleState object         CirclePanel object
  |  about the      |      +-----------------+       +------------------+
  |  keyboard event |      |  int xNew       |       | CircleState cs   |
  +-----------------+      |  int yNew       |       |                  |
                           |  int xOld       |       |                  |
                           |  int yOld       |       | paintComponent() |
                           +-----------------+       +------------------+
                                                     GUI object that needs
                                                     to be repainted after
                                                     an event occurs.
*/
