/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   In this version we combine the listener object
   and the GUI panel object with the program object.
   In other words, our program IS both the JPanel and
   the listener object. This gives the listener easy
   access to the JPanel and data objects, and it gives
   the JPanle easy access to the data object.
*/
@SuppressWarnings("serial")
public class CircleProgram extends JPanel implements KeyListener
{
   private final CircleState cs;

   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      // Set the preferred size of this JPanel.
      this.setPreferredSize(new Dimension(600, 600));

      // Now create the JFrame that will hold this JPanel
      final JFrame jf = new JFrame("Interactive Frame v3b");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      cs = new CircleState();
      jf.getContentPane().add(this);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.setVisible(true);

      // This object is the event listener for
      // key events coming from the JFrame.
      jf.addKeyListener(this);
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
   }//main()

   @Override
   public void paintComponent(Graphics g)
   {
      g.setColor(Color.white);
      g.fillOval(cs.xOld, cs.yOld, 20, 20); // erase old disk
      g.setColor(Color.red);
      g.fillOval(cs.xNew, cs.yNew, 20, 20); // draw new disk
      cs.xOld = cs.xNew;
      cs.yOld = cs.yNew;
   }


   // implement the KeyListener interface (three methods)
   @Override public void keyPressed(KeyEvent e)
   {
      final int keyCode = e.getKeyCode();
      if (keyCode == KeyEvent.VK_UP)
      {
         cs.yNew -= 5;  // update the state
         repaint();     // notify the view
      }
      else if (keyCode == KeyEvent.VK_DOWN)
      {
         cs.yNew += 5;
         repaint();
      }
      else if (keyCode == KeyEvent.VK_LEFT)
      {
         cs.xNew -= 5;
         repaint();
      }
      else if (keyCode == KeyEvent.VK_RIGHT)
      {
         cs.xNew += 5;
         repaint();
      }
   }
   @Override public void keyReleased(KeyEvent e){ }
   @Override public void keyTyped(KeyEvent e){ }
}//CircleProgram

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+
|                      |    List<KeyListener>     CircleProgram object
|  List<KeyListener>---|------->+------+        +-------------------------+
|                      |        |      |        | CircleState cs          |
|                      |        +------+        |                         |
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
GUI object that receives        |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM.                                        |                         |
                                                | paintComponent()        |
                                                +-------------------------+
                                                  An object that implements
    KeyEvent object       CircleState object      the KeyListener interface
  +-----------------+     +-----------------+     and is a subclass of JPanel.
  |  information    |     |  int xNew       |     It needs to be repainted
  |  from the OS    |     |  int yNew       |     after an event occurs.
  |  about the      |     |  int xOld       |
  |  keyboard event |     |  int yOld       |
  +-----------------+     +-----------------+

*/
