/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   In this version we combine the listener object,
   the GUI panel object, and the data object into
   one single object, which is an instance of the
   program object. So the CircleProgram class is
   taking on all the responsibilities of this program.
   This makes for a very compact, if a bit confusing,
   design.
*/
@SuppressWarnings("serial")
public class CircleProgram extends JPanel implements KeyListener
{
   private int xNew = 300; // where the circle needs to be
   private int yNew = 300;
   private int xOld = 300; // where the circle is now
   private int yOld = 300;

   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      // Set the preferred size of this JPanel.
      this.setPreferredSize(new Dimension(600, 600));

      // Now create the JFrame that will hold this JPanel
      final JFrame jf = new JFrame("Interactive JFrame v3c");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jf.getContentPane().add(this);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.setVisible(true);

      // This object is the event listener for
      // key events coming from the JFrame.
      jf.addKeyListener(this);
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
   }//main()


   @Override
   public void paintComponent(Graphics g)
   {
      g.setColor(Color.white);
      g.fillOval(xOld, yOld, 20, 20); // erase old disk
      g.setColor(Color.red);
      g.fillOval(xNew, yNew, 20, 20); // draw new disk
      xOld = xNew;
      yOld = yNew;
   }


   // implement the KeyListener interface (three methods)
   @Override public void keyPressed(KeyEvent e)
   {
      final int keyCode = e.getKeyCode();
      if (keyCode == KeyEvent.VK_UP)
      {
         yNew -= 5;  // update the state
         repaint();  // notify the view
      }
      else if (keyCode == KeyEvent.VK_DOWN)
      {
         yNew += 5;
         repaint();
      }
      else if (keyCode == KeyEvent.VK_LEFT)
      {
         xNew -= 5;
         repaint();
      }
      else if (keyCode == KeyEvent.VK_RIGHT)
      {
         xNew += 5;
         repaint();
      }
   }
   @Override public void keyReleased(KeyEvent e){ }
   @Override public void keyTyped(KeyEvent e){ }
}//CircleProgram

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+
|                      |       KeyListener         CircleProgram object
|  List<KeyListener>---|------->+------+        +-------------------------+
|                      |        |      |        | int xNew                |
|                      |        +------+        | int yNew                |
|  addKeyListener()    |        |  ----|------->| int xOld                |
+----------------------+        +------+        | int yOld                |
GUI object that receives        |      |        |                         |
KeyEvent objects from           +------+        | keyPressed(KeyEvent e)  |
the JVM                                         | keyReleased(KeyEvent e) |
                                                | keyTyped(KeyEvent e)    |
             KeyEvent object                    |                         |
           +-----------------+                  | paintComponent()        |
           |  information    |                  +-------------------------+
           |  from the OS    |                   GUI object that implements
           |  about the      |                   the KeyListener interface
           |  keyboard event |                   and is a subclass of JPanel
           +-----------------+
*/
