/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   The event handler class is a static nested class
   of our program class. The event handler object has
   access to the program class's private static variables.
   (This is NOT a common way to write an event handler.)
*/
public class CircleProgram
{
   public  static CircleState cs; // Compare with CircleProgram, ver 1b.
   private static CirclePanel cp;

   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      final JFrame jf = new JFrame("Interactive Frame v4");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      cs = new CircleState();
      cp = new CirclePanel();
      jf.getContentPane().add(cp);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.setVisible(true);
      jf.addKeyListener(new CircleListener());
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
   }//main()


   // A static nested class definition.
   static class CircleListener implements KeyListener
   {
      @Override public void keyPressed(KeyEvent e)
      {
         final int keyCode = e.getKeyCode();
         if (keyCode == KeyEvent.VK_UP)
         {
            cs.yNew -= 5;  // update the state
            cp.repaint();  // notify the view
         }
         else if (keyCode == KeyEvent.VK_DOWN)
         {
            cs.yNew += 5;
            cp.repaint();
         }
         else if (keyCode == KeyEvent.VK_LEFT)
         {
            cs.xNew -= 5;
            cp.repaint();
         }
         else if (keyCode == KeyEvent.VK_RIGHT)
         {
            cs.xNew += 5;
            cp.repaint();
         }
      }
      @Override public void keyReleased(KeyEvent e){ }
      @Override public void keyTyped(KeyEvent e){ }
   }//CircleListener
}//CircleProgram

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                                   CircleProgram class
                                               +=============================+
                                              / public  static CircleState cs \
                                              \ private static CirclePanel cp /
                                              /                               \
      JFrame object                           \ public static main()          /
+----------------------+                      /                               \
|                      |   List<KeyListener>  \                               /
|  List<KeyListener>---|------->+------+      / static CircleListener object  \
|                      |        |      |      \ +-------------------------+   /
|                      |        +------+      / |                         |   \
|  addKeyListener()    |        |  ----|------->| keyPressed(KeyEvent e)  |   /
+----------------------+        +------+      / | keyReleased(KeyEvent e) |   \
GUI object that receives        |      |      \ | keyTyped(KeyEvent e)    |   /
KeyEvent objects from           +------+      / |                         |   \
the JVM.                                      \ +-------------------------+   /
                                              /  An object that implements    \
                                              \  the KeyListener interface.   /
                                              /                               \
                                             +================================+

    KeyEvent object         CircleState object       CirclePanel object
  +-----------------+      +-----------------+      +------------------+
  |  information    |      |  int xNew       |      |                  |
  |  from the OS    |      |  int yNew       |      |                  |
  |  about the      |      |  int xOld       |      |                  |
  |  keyboard event |      |  int yOld       |      | paintComponent() |
  +-----------------+      +-----------------+      +------------------+
                                                    GUI object that needs
                                                    to be repainted after
                                                    an event occurs.
*/
